Sound={tracks:{},_enabled:!0,template:new Template('<embed style="height:0" id="sound_#{track}_#{id}" src="#{url}" loop="false" autostart="true" hidden="true"/>'),enable:function(){Sound._enabled=!0},disable:function(){Sound._enabled=!1},play:function(e){if(Sound._enabled){var t=Object.extend({track:"global",url:e,replace:!1},arguments[1]||{});t.replace&&this.tracks[t.track]&&($R(0,this.tracks[t.track].id).each(function(e){var i=$("sound_"+t.track+"_"+e);i.Stop&&i.Stop(),i.remove()}),this.tracks[t.track]=null),this.tracks[t.track]?this.tracks[t.track].id++:this.tracks[t.track]={id:0},t.id=this.tracks[t.track].id,$$("body")[0].insert(Prototype.Browser.IE?new Element("bgsound",{id:"sound_"+t.track+"_"+t.id,src:t.url,loop:1,autostart:!0}):Sound.template.evaluate(t))}}},Prototype.Browser.Gecko&&navigator.userAgent.indexOf("Win")>0&&(navigator.plugins&&$A(navigator.plugins).detect(function(e){return-1!=e.name.indexOf("QuickTime")})?Sound.template=new Template('<object id="sound_#{track}_#{id}" width="0" height="0" type="audio/mpeg" data="#{url}"/>'):navigator.plugins&&$A(navigator.plugins).detect(function(e){return-1!=e.name.indexOf("Windows Media")})?Sound.template=new Template('<object id="sound_#{track}_#{id}" type="application/x-mplayer2" data="#{url}"></object>'):navigator.plugins&&$A(navigator.plugins).detect(function(e){return-1!=e.name.indexOf("RealPlayer")})?Sound.template=new Template('<embed type="audio/x-pn-realaudio-plugin" style="height:0" id="sound_#{track}_#{id}" src="#{url}" loop="false" autostart="true" hidden="true"/>'):Sound.play=function(){});