body {
	background: #eee;
	font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
	font-size: 12px;
	margin: 0;
	display: inline-block;
    width: 100%;
}

h1 {
	font-size: 32px;
	line-height:32px;
	font-weight: 400;
	margin: 10px 5px;
}
h2 {
	font-size: 16px;
	line-height:16px;
	font-weight: 400;
	margin: 5px;
}
small {
	font-size: 10px;
}
hr {
	display: none;
}
img {
	border: 0;
}

.sidebar {
	background: #4c4c4c;
	position: fixed;
	top: 0;
	left: 0;
	width: 250px;
	height: 100%;
	color: #ccc;
}

.sidebar-header {
	border-bottom: 1px solid #373737;
	height: 50px;
	position: relative;
	padding: 10px;
	box-sizing: border-box;
}
.sidebar-header .logo {
	margin: 0;
	display: inline-block;
	padding: 0 5px;
}

.sidebar-menu {
	overflow-x: auto;
	overflow-y: auto;
	position: absolute;
	top: 51px;
	bottom: 0px;
	width: 249px;
}

.sidebar-menu img {
	margin-left: 15px;
}

.sidebar .filter-label,
.sidebar .hierarchy-label {
	display: none;
}
.sidebar .filter-text,
.sidebar .hierarchy-popup {
	margin: 10px;
}
.sidebar .hierarchy-popup select {
	width: 100%;
}
.sidebar .filter-text input {
	width: 100%;
	box-sizing: border-box;
	border: 0;
	padding: 5px;
	border-radius: 3px;
	background-color: #232323;
	color: #ccc;
}

.sidebar ul.menu {
	margin: 0;
	padding: 0;
	list-style: none;
	display: block;
	padding-bottom: 10px;
}

.sidebar ul.menu li {
	margin: 2px 10px;
	border: 1px solid transparent;
	position: relative;
}
.sidebar ul.menu li.menuopen,
.sidebar ul.menu li:hover,
.sidebar ul.menu li.menuactive {
	border: 1px solid #4a4a4a;
	background: #232323;
	color: #fff;
}
.sidebar ul.menu li a {
	color: #ccc;
	line-height: 2.0em;
	display: block;
	padding-left: 5px;
	text-decoration: none;
}
.sidebar ul.menu li a.menulinkactive {
	color: #67c2ef;
}
.sidebar ul.menu li ul {
	background: #343434;
	padding: 5px;
}


.navbar {
	height: 50px;
	border-bottom: 1px solid #afafaf;
	background: linear-gradient(#f8f8f8,#eee);
	box-sizing: border-box;
	position: fixed;
	top: 0;
	right: 0;
	left: 250px;
	box-shadow: 0 3px 1px -1px rgba(0,0,0,0.05);
	text-shadow: 0 1px 1px rgba(255,255,255,0.9);
}

.navbar .navbar-menu {
	border-right: 1px solid #afafaf;
	height: 50px;
	display: inline-block;
}
.navbar .navbar-refresh {
	border-right: 1px solid #afafaf;
	height: 50px;
	display: inline-block;
	position: absolute;
	padding: 0 10px;
	line-height: 50px;
	font-size: 14px;
	font-weight: 500;
	cursor: pointer;
}
.navbar .navbar-user {
	border-left: 1px solid #afafaf;
	height: 50px;
	display: inline-block;
	right: 0;
	position: absolute;
	padding: 0 10px;
	line-height: 50px;
	font-size: 14px;
	font-weight: 500;
}
.navbar .navbar-user .icon-person {
	background: no-repeat left url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACQAAAAkCAQAAABLCVATAAAA1ElEQVR4Ae3SPUoDYRSG0QPqDGQD4v9qdAu6BBGNcQO6FME+lbgCLSQhEqMb0EK7QSRoCuFqKcOHzIyIzZynf+HC1fqlXM/A1MzIkVxDaybiWxMrGshNRKmxTG09kWhPbQOR6EptU5Go+Meh9GmXauuKRLtqy92KUjcWNLBqXPqiZQ1lDlx79WboUKb1Jzq2nOi7V/j4qnCn79imjooyOy7MhHTvzm3L/Ghe17Oo0JN9c6RtGIkaDa1LWPIoavZgkbIz0aBTyl5EgwrKolntUIUqa30CLm2RNE9+n04AAAAASUVORK5CYII=');
	width: 32px;
	height: 32px;
	display: inline-block;
	vertical-align: middle;
	padding-right: 10px;
	opacity: 0.3;
}
a#menu-button {
	background: no-repeat center url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACQAAAAkCAQAAABLCVATAAAAJUlEQVR4AWOgKhgFo+A/eXAIGIQAo2A0HY2mo9F0NJqORsEoAAAO8PsFZyNgiAAAAABJRU5ErkJggg==);
	text-indent: 100%;
	white-space: nowrap;
	overflow: hidden;
	display: block;
	opacity: 0.3;
	height: 50px;
	width: 50px;
}
a#menu-button:hover {
	opacity: 0.5;
}

.main {
	padding-left: 250px;
	margin: 10px 10px;
	padding-top: 50px;
	display: inline-block;
}

.main .panel, .main .panel-no-border {
	margin: 5px;
	display: inline-block;
}

.main .panel {
	border: 1px solid #d4d4d4;
	box-shadow: 0 2px 1px -1px rgba(0,0,0,0.1);
	background: #fff;
}

.main .panel-heading {
	background: #f5f5f5;
	border-bottom: 1px solid #d4d4d4;
	box-shadow: inset 0 0 1px 1px #fff;
}

.panel-heading-no-border {
}

.main .panel-heading h2, .main .panel-heading-no-border h2 {
	margin: 0;
	padding-left: 10px;
	line-height: 30px;
	font-size: 14px;
	font-weight: 500;
}

.main .panel-body {
	padding: 5px;
}

.footer {
	border-top: 1px solid #d4d4d4;
	padding-left: 250px;
}

.footer p {
	display: inline-block;
	margin: 5px 10px;
}

.footer p.footer-right {
	float: right;
}

.footer,
.footer a {
	color: #747474;
	text-decoration: none;
}
.footer a:hover {
	text-decoration: underline;
}

.sidebar-hidden .sidebar {
	left: -250px !important;
}

.sidebar-hidden .main,
.sidebar-hidden .footer {
	padding-left: 0 !important;
}
.sidebar-hidden .navbar {
	left: 0 !important;
}

@media (max-width: 900px) {
	.sidebar, .navbar {
		transition: left 0.4s ease;
	}
	.sidebar {
		left: -250px;
	}
	.navbar {
		left: 0;
	}
	.main, .footer {
		padding-left: 0;
	}
	.sidebar-visible .sidebar {
		left: 0px !important;
	}
	.sidebar-visible .navbar {
		left: 250px !important;
	}
}

.img-responsive {
    max-width: 100%;
    height: auto;
}
