body, html {
	font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
	font-size: 12px;
}
img {
	border: 0;
}
.sidebar, .navbar {
	display: none;
}
.panel {
	page-break-inside: avoid;
}
.panel h2 {
	font-size: 12px;
}
